/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2002 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <sys/types.h>	/* clex.h */
#include <stdio.h>		/* puts() */
#include <stdlib.h>		/* putenv() */
#include <string.h>		/* strcmp() */
#include <unistd.h>		/* chdir() */

#include "start.h"
#include "clex.h"

#include "cfg.h"		/* config_initialize() */
#include "completion.h"	/* completion_initialize() */
#include "control.h"	/* control_loop() */
#include "directory.h"	/* dir_initialize() */
#include "err.h"		/* err_initialize() */
#include "exec.h"		/* exec_initialize() */
#include "filepanel.h"	/* files_initialize() */
#include "help.h"		/* help_initialize() */
#include "history.h"	/* hist_initialize() */
#include "inout.h"		/* curses_initialize() */
#include "locate.h"		/* locate_initialize() */
#include "list.h"		/* list_initialize() */
#include "select.h"		/* select_initialize() */
#include "sort.h"		/* sort_initialize() */
#include "tty.h"		/* tty_initialize() */
#include "userdata.h"	/* userdata_initialize() */
#include "ustring.h"	/* get_cwd_us() */
#include "util.h"		/* base_name() */
#include "xterm_title.h"/* xterm_title_initialize() */

/* GLOBAL VARIABLES */
PANEL *panel = 0;
PANEL_COMPLETION panel_compl;
PANEL_CONFIG panel_config;
PANEL_DIR_BROWSE panel_dir_browse;
PANEL_DIR_SPLIT panel_dir_split;
PANEL_FILE *ppanel_file;
PANEL_HELP panel_help;
PANEL_HISTORY panel_hist;
PANEL_LOCATE panel_locate;
PANEL_MENU panel_menu, panel_config_confirm;
PANEL_SORT panel_sort;
DISPLAY display = { 0,0,0,0 };
TEXTLINE *textline = 0, line_cmd, line_tmp, line_dir;
int admin = 0;
int next_mode;

static USTRING cwd0 = { 0,0 };
static const char *argv0;
static int warnings = 0;

#define MAX_WARNINGS	12

/* start CLEX in eleven easy steps */
int
main(int argc, char *argv[])
{
	err_initialize();	/* #1 - before anything that might fail */
	tty_initialize();	/* #2 - before any I/O */

	/* #3 - initialize miscellaneous variables */
#ifdef HAVE_PUTENV
	putenv("CLEX=" VERSION);/* let's have $CLEX (just so) */
#endif
	US_INIT(line_tmp.line);	/* belongs to nowhere, initialize it here */
	/* save path for possible restart */
	argv0 = argv[0];
	if (get_cwd_us(&cwd0) < 0)
		argv0 = 0;

	/* #4 - say hello */
	puts("\n\n\n\nStarting CLEX " VERSION "\n");
	fflush(stdout);

	/* #5 - check command line arguments */
	if (argc > 1) {
		if (argc == 2 && strcmp(argv[1],"-a") == 0)
			admin = 1;	/* configure defaults and nothing else */
		else
			err_exit("Usage: %s [-a]\n",base_name(argv0));
	}

	userdata_initialize();	/* #6 - before config */
	config_initialize();	/* #7 - read configuration */

	/* #8 - initialize the rest, the order is no longer important */
	help_initialize();
	if (!admin) {
		/* all these features are not accessed in admin mode */
		completion_initialize();
		dir_initialize();
		exec_initialize();
		files_initialize();
		hist_initialize();
		list_initialize();
		locate_initialize();
		menu_initialize();
		select_initialize();
		sort_initialize();
		xterm_title_initialize();
		xterm_title_set();
	}

	/* #9 - last chance to write normally to the screen */
	if (warnings) {
		printf("\nPress <Enter> to continue ");
		fflush(stdout);
		getchar();
	}
	warnings = -1;	/* end of startup */

	/* #10 - start CURSES screen handling */
	curses_initialize();

	/* #11 - let's go ! */
	control_loop(admin ? MODE_CONFIG_ADMIN : MODE_FILE_MAIN);

	/* NOTREACHED */
	return 0;
}

static void
restart(void)
{
	/* registered by atexit(), invoked just before exit */
	if (argv0 && chdir(USTR(cwd0)) == 0)
		execl(argv0,argv0,(char *)0);
	puts("cannot restart CLEX");
}

void
restart_clex(void)
{
	atexit(restart);
	err_exit("restarting CLEX");
}

void
startup_warning(void)
{
	fflush(stdout);
	if (warnings >= 0 && ++warnings > MAX_WARNINGS)
		err_exit("Too many errors during startup");
}
