/*
 *
 * CLEX File Manager
 *
 * Copyright (C) 1997-2000 Tempest s.r.o. http://www.tempest.sk
 * Copyright (C) 2001-2002 Vlado Potisk <vlado_potisk@clex.sk>
 *
 * CLEX is free software without warranty of any kind; see the
 * GNU General Public License as set out in the "COPYING" document
 * which accompanies the CLEX File Manager package.
 *
 * CLEX can be downloaded from http://www.clex.sk
 *
 */

#include <autoconf.h>

#include <stdio.h>		/* setvbuf() */
#include <termios.h>	/* struct termios */
#include <unistd.h>		/* STDIN_FILENO */

#include "tty.h"

#include "err.h"		/* err_exit() */

static struct termios save_termios;

void tty_initialize(void)
{
	if (tcgetattr(STDIN_FILENO,&save_termios) == -1)
		err_exit("Can't get the terminal settings");
#ifndef SETVBUF_REVERSED
	setvbuf(stdin,0,_IONBF,0);	/* input unbuffered */
	setvbuf(stdout,0,_IOFBF,0);	/* output fully buffered */
#else
	setvbuf(stdin,_IONBF,0,0);
	setvbuf(stdout,_IOFBF,0,0);
#endif
}

void tty_setraw(void)	/* noncanonical, no echo */
{
	struct termios buf;

	buf = save_termios;
	buf.c_lflag &= ~(ECHO | ICANON | ISIG | IEXTEN);
	buf.c_cc[VMIN] = 1;
	buf.c_cc[VTIME] = 0;
	tcsetattr(STDIN_FILENO,TCSAFLUSH,&buf);
}

void tty_reset(void)
{
	tcsetattr(STDIN_FILENO,TCSAFLUSH,&save_termios);
}
